@@ -39,12 +39,22 @@ try:
 except ImportError:
     KIWI_AVAILABLE = False
 
+try:
+    import openai
+    from .openai_similarity import OpenAISimilarityCalculator
+    OPENAI_AVAILABLE = True
+except ImportError:
+    OPENAI_AVAILABLE = False
+
 
 def evaluate_with_ranx_similarity(retriever, questions: List[str], 
                                  reference_contexts: List[List[str]], 
                                  k: int = 5,
                                  method: str = 'embedding', 
-                                 similarity_threshold: float = 0.7) -> Dict[str, float]:
+                                 similarity_threshold: float = 0.7,
+                                 embedding_model: str = "sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2",
+                                 use_graded_relevance: bool = False,
+                                 evaluation_mode: str = 'reference_based') -> Dict[str, float]:
     """
     Evaluate RAG system using ranx with semantic similarity matching.
     
@@ -57,8 +67,13 @@ def evaluate_with_ranx_similarity(retriever, questions: List[str],
         questions: List of questions to evaluate.
         reference_contexts: List of reference document lists for each question.
         k: Number of top retrieved documents to evaluate.
-        method: Similarity calculation method ('embedding', 'rouge', 'kiwi_rouge').
+        method: Similarity calculation method ('embedding', 'rouge', 'kiwi_rouge', 'openai').
         similarity_threshold: Minimum similarity score to consider relevant (0.0-1.0).
+        embedding_model: Sentence transformer model name for embedding method.
+        use_graded_relevance: If True, use similarity scores as relevance grades. 
+                             If False, use binary relevance (0 or 1).
+        evaluation_mode: 'reference_based' evaluates against all reference documents,
+                        'retrieval_based' evaluates only retrieved documents.
         
     Returns:
         Dictionary containing ranx evaluation metrics:
@@ -67,18 +82,78 @@ def evaluate_with_ranx_similarity(retriever, questions: List[str],
         - map@k: Mean Average Precision at k  
         - mrr: Mean Reciprocal Rank
         
-    Example:
-        >>> from krag.evaluation import evaluate_with_ranx_similarity
+    Examples:
+        Basic usage with default embedding model:
+        >>> from ranx_k.evaluation import evaluate_with_ranx_similarity
         >>> results = evaluate_with_ranx_similarity(
         ...     retriever=my_retriever,
         ...     questions=["RAG 시스템이란?"],
         ...     reference_contexts=[["RAG는 검색 증강 생성..."]], 
         ...     k=5,
-        ...     method='kiwi_rouge',
+        ...     method='embedding',
         ...     similarity_threshold=0.6
         ... )
         >>> print(f"Hit@5: {results['hit_rate@5']:.3f}")
         
+        Using different embedding models:
+        >>> # More accurate multilingual model
+        >>> results = evaluate_with_ranx_similarity(
+        ...     retriever=my_retriever,
+        ...     questions=questions,
+        ...     reference_contexts=references,
+        ...     method='embedding',
+        ...     similarity_threshold=0.7,
+        ...     embedding_model="sentence-transformers/paraphrase-multilingual-mpnet-base-v2"
+        ... )
+        
+        >>> # Latest BGE-M3 model
+        >>> results = evaluate_with_ranx_similarity(
+        ...     retriever=my_retriever,
+        ...     questions=questions,
+        ...     reference_contexts=references,
+        ...     method='embedding',
+        ...     embedding_model="BAAI/bge-m3"
+        ... )
+        
+        Reference-based evaluation (properly calculates recall):
+        >>> results = evaluate_with_ranx_similarity(
+        ...     retriever=my_retriever,
+        ...     questions=questions,
+        ...     reference_contexts=references,
+        ...     method='kiwi_rouge',
+        ...     similarity_threshold=0.3,
+        ...     evaluation_mode='reference_based'  # Evaluates against all reference docs
+        ... )
+        
+        Using graded relevance instead of binary:
+        >>> results = evaluate_with_ranx_similarity(
+        ...     retriever=my_retriever,
+        ...     questions=questions,
+        ...     reference_contexts=references,
+        ...     method='embedding',
+        ...     use_graded_relevance=True,  # Uses similarity scores as relevance grades
+        ...     evaluation_mode='reference_based'
+        ... )
+        
+        Using Korean-optimized Kiwi ROUGE:
+        >>> results = evaluate_with_ranx_similarity(
+        ...     retriever=my_retriever,
+        ...     questions=questions,
+        ...     reference_contexts=references,
+        ...     method='kiwi_rouge',
+        ...     similarity_threshold=0.3  # Lower threshold for Kiwi ROUGE
+        ... )
+        
+        Using OpenAI Embeddings:
+        >>> results = evaluate_with_ranx_similarity(
+        ...     retriever=my_retriever,
+        ...     questions=questions,
+        ...     reference_contexts=references,
+        ...     method='openai',
+        ...     similarity_threshold=0.7,
+        ...     embedding_model="text-embedding-3-small"  # OpenAI model name
+        ... )
+        
     Raises:
         ImportError: If required dependencies are not installed.
     """
@@ -87,7 +162,8 @@ def evaluate_with_ranx_similarity(retriever, questions: List[str],
             "ranx is required for this evaluation. Install with: pip install ranx"
         )
     
-    print(f"🔍 유사도 기반 ranx 평가 시작 (방법: {method}, 임계값: {similarity_threshold})")
+    print(f"🔍 Starting similarity-based ranx evaluation | 유사도 기반 ranx 평가 시작")
+    print(f"   Method | 방법: {method}, Threshold | 임계값: {similarity_threshold}, Mode | 모드: {evaluation_mode}")
     
     # Initialize similarity calculator based on method
     if method == 'embedding':
@@ -96,7 +172,7 @@ def evaluate_with_ranx_similarity(retriever, questions: List[str],
                 "sentence-transformers and scikit-learn required for embedding method. "
                 "Install with: pip install sentence-transformers scikit-learn"
             )
-        similarity_calculator = EmbeddingSimilarityCalculator()
+        similarity_calculator = EmbeddingSimilarityCalculator(model_name=embedding_model)
     elif method == 'rouge':
         if not ROUGE_AVAILABLE:
             raise ImportError(
@@ -109,19 +185,45 @@ def evaluate_with_ranx_similarity(retriever, questions: List[str],
                 "kiwipiepy required for kiwi_rouge method. Install with: pip install kiwipiepy"
             )
         similarity_calculator = KiwiRougeSimilarityCalculator()
+    elif method == 'openai':
+        if not OPENAI_AVAILABLE:
+            raise ImportError(
+                "openai required for openai method. Install with: pip install openai"
+            )
+        similarity_calculator = OpenAISimilarityCalculator(model_name=embedding_model)
     else:
         raise ValueError(f"Unsupported similarity method: {method}")
     
     qrels_dict = {}
     run_dict = {}
     
-    for i, (question, ref_docs) in tqdm(enumerate(zip(questions, reference_contexts)), 
-                                       desc="ranx 유사도 평가"):
+    # Track total retrieved documents
+    total_retrieved_docs = 0
+    
+    # Create iterator that shows progress bar only after first 3 iterations
+    iterator = enumerate(zip(questions, reference_contexts))
+    total_questions = len(questions)
+    
+    # Initialize progress bar as None
+    pbar = None
+    
+    for i, (question, ref_docs) in iterator:
+        # Create progress bar after first 3 iterations
+        if i == 3 and pbar is None:
+            pbar = tqdm(total=total_questions, 
+                       desc="ranx similarity evaluation | ranx 유사도 평가",
+                       initial=3)  # Start from 3 since we've already done 3
+        
+        # Update progress bar if it exists
+        if pbar is not None:
+            pbar.update(1)
+        
         query_id = f"q_{i+1}"
         
         # Retrieve documents
         retrieved_docs = retriever.invoke(question)[:k]
         retrieved_texts = [doc.page_content for doc in retrieved_docs]
+        total_retrieved_docs += len(retrieved_docs)
         
         # Extract reference texts
         ref_texts = []
@@ -134,33 +236,85 @@ def evaluate_with_ranx_similarity(retriever, questions: List[str],
             ref_texts, retrieved_texts
         )
         
-        # Build qrels: relevance judgments for reference documents
+        # Build qrels and run based on evaluation mode
         qrels_dict[query_id] = {}
-        for j, ref_text in enumerate(ref_texts):
-            ref_id = f"ref_{j}"
-            # Maximum similarity with any retrieved document  
-            max_similarity = np.max(similarity_matrix[j, :]) if similarity_matrix.shape[1] > 0 else 0
-            
-            # Only include if above threshold
-            if max_similarity >= similarity_threshold:
-                qrels_dict[query_id][ref_id] = float(max_similarity)
-        
-        # Build run: scores for retrieved documents
         run_dict[query_id] = {}
-        for j, ret_text in enumerate(retrieved_texts):
-            ret_id = f"ret_{j}"
-            # Maximum similarity with any reference document
-            max_similarity = np.max(similarity_matrix[:, j]) if similarity_matrix.shape[0] > 0 else 0
-            run_dict[query_id][ret_id] = float(max_similarity)
+        
+        if evaluation_mode == 'reference_based':
+            # Reference-based evaluation: Include all reference documents in qrels
+            # This properly evaluates recall
+            
+            # First, add all reference documents to qrels
+            for ref_idx, ref_text in enumerate(ref_texts):
+                ref_doc_id = f"ref_{ref_idx}"
+                if use_graded_relevance:
+                    qrels_dict[query_id][ref_doc_id] = 1.0  # All reference docs have relevance 1.0
+                else:
+                    qrels_dict[query_id][ref_doc_id] = 1.0
+            
+            # Then, check which reference documents were retrieved
+            for ref_idx, ref_text in enumerate(ref_texts):
+                ref_doc_id = f"ref_{ref_idx}"
+                
+                # Find best matching retrieved document for this reference
+                if similarity_matrix.shape[1] > 0:
+                    similarities = similarity_matrix[ref_idx, :]
+                    best_match_idx = np.argmax(similarities)
+                    best_similarity = similarities[best_match_idx]
+                    
+                    # Add to run if similarity is above threshold
+                    if best_similarity >= similarity_threshold:
+                        run_dict[query_id][ref_doc_id] = float(best_similarity)
+                        
+            # Also add non-reference retrieved documents with low scores
+            for j, ret_text in enumerate(retrieved_texts):
+                # Check if this retrieved doc matches any reference
+                if similarity_matrix.shape[0] > 0:
+                    max_sim_to_any_ref = np.max(similarity_matrix[:, j])
+                    if max_sim_to_any_ref < similarity_threshold:
+                        # This is a non-relevant retrieved document
+                        non_ref_id = f"non_ref_{j}"
+                        run_dict[query_id][non_ref_id] = 0.0
+                        
+        else:
+            # Retrieval-based evaluation: Original behavior
+            for j, ret_text in enumerate(retrieved_texts):
+                doc_id = f"doc_{j}"
+                
+                # Find maximum similarity with any reference document
+                max_similarity = np.max(similarity_matrix[:, j]) if similarity_matrix.shape[0] > 0 else 0
+                
+                # Add to run (all retrieved documents with their similarity scores)
+                run_dict[query_id][doc_id] = float(max_similarity)
+                
+                # Add to qrels based on relevance
+                if use_graded_relevance:
+                    # Use actual similarity score as relevance grade
+                    if max_similarity >= similarity_threshold:
+                        qrels_dict[query_id][doc_id] = float(max_similarity)
+                else:
+                    # Binary relevance
+                    if max_similarity >= similarity_threshold:
+                        qrels_dict[query_id][doc_id] = 1.0
         
         # Debug info for first few queries
         if i < 3:
-            print(f"\n🔍 질문 {i+1}: {question}")
-            print(f"📊 유사도 매트릭스 shape: {similarity_matrix.shape}")
-            print(f"📈 최대 유사도: {np.max(similarity_matrix):.3f}")
-            print(f"📋 qrels 항목 수: {len(qrels_dict[query_id])}")
+            print(f"\n🔍 Question | 질문 {i+1}: {question[:50]}...")
+            print(f"📊 Reference docs | 참조 문서: {len(ref_texts)}, Retrieved docs | 검색 문서: {len(retrieved_texts)}")
+            print(f"📊 Similarity matrix shape | 유사도 매트릭스 shape: {similarity_matrix.shape}")
+            if similarity_matrix.size > 0:
+                print(f"📈 Max similarity | 최대 유사도: {np.max(similarity_matrix):.3f}")
+            print(f"📋 Qrels items | qrels 항목 수: {len(qrels_dict[query_id])}")
+            print(f"📋 Run items | run 항목 수: {len(run_dict[query_id])}")
+            if evaluation_mode == 'reference_based':
+                retrieved_count = len([doc for doc in run_dict[query_id] if doc.startswith('ref_')])
+                print(f"📋 Retrieved reference docs | 검색된 참조 문서: {retrieved_count}/{len(ref_texts)}")
             print("-" * 50)
     
+    # Close progress bar if it was created
+    if pbar is not None:
+        pbar.close()
+    
     # Evaluate using ranx
     try:
         qrels = Qrels(qrels_dict)
@@ -170,7 +324,7 @@ def evaluate_with_ranx_similarity(retriever, questions: List[str],
         metrics = [f"hit_rate@{k}", f"ndcg@{k}", f"map@{k}", "mrr"]
         results = evaluate(qrels, run, metrics)
         
-        print(f"\n📊 유사도 기반 ranx 평가 결과 ({method}):")
+        print(f"\n📊 Similarity-based ranx evaluation results | 유사도 기반 ranx 평가 결과 ({method}):")
         for metric, score in results.items():
             print(f"  {metric}: {score:.3f}")
         
@@ -179,11 +333,20 @@ def evaluate_with_ranx_similarity(retriever, questions: List[str],
         total_queries = len(qrels_dict)
         avg_relevant_per_query = total_relevant_docs / total_queries if total_queries > 0 else 0
         
-        print(f"\n📈 분석 정보:")
-        print(f"  총 질문 수: {total_queries}")
-        print(f"  관련 문서 총 개수: {total_relevant_docs}")
-        print(f"  질문당 평균 관련 문서: {avg_relevant_per_query:.1f}")
-        print(f"  사용된 임계값: {similarity_threshold}")
+        print(f"\n📈 Analysis information | 분석 정보:")
+        print(f"  Total queries | 총 질문 수: {total_queries}")
+        print(f"  Total retrieved docs | 총 검색 문서 수: {total_retrieved_docs}")
+        print(f"  Avg docs per query | 질문당 평균 검색 문서: {total_retrieved_docs/total_queries:.1f}")
+        print(f"  Total relevant docs | 관련 문서 총 개수: {total_relevant_docs}")
+        print(f"  Avg relevant per query | 질문당 평균 관련 문서: {avg_relevant_per_query:.1f}")
+        print(f"  Threshold used | 사용된 임계값: {similarity_threshold}")
+        
+        if evaluation_mode == 'reference_based':
+            total_retrieved = sum(len([doc for doc in run[query_id] if doc.startswith('ref_')]) 
+                                for query_id in run_dict)
+            total_reference = sum(len(ref_docs) for ref_docs in reference_contexts)
+            overall_recall = total_retrieved / total_reference if total_reference > 0 else 0
+            print(f"  Overall recall | 전체 재현율: {overall_recall:.3f} ({total_retrieved}/{total_reference})")
         
         return results
         
